/* -*- c++ -*- ----------------------------------------------------------
   LAMMPS - Large-scale Atomic/Molecular Massively Parallel Simulator
   http://lammps.sandia.gov, Sandia National Laboratories
   Steve Plimpton, sjplimp@sandia.gov

   Copyright (2003) Sandia Corporation.  Under the terms of Contract
   DE-AC04-94AL85000 with Sandia Corporation, the U.S. Government retains
   certain rights in this software.  This software is distributed under
   the GNU General Public License.

   See the README file in the top-level LAMMPS directory.
------------------------------------------------------------------------- */

#ifdef FIX_CLASS

FixStyle(rattle,FixRattle)

#else

#ifndef LMP_FIX_RATTLE_H
#define LMP_FIX_RATTLE_H

#include "fix.h"
#include "fix_shake.h"
namespace LAMMPS_NS {


// For testing and debugging. Set RATTLE_TEST to 1 if you want the contraints to be checked
// at the end of the time step. Set RATTLE_DEBUG to 1 if you want some more output for debuggin purposes.
#define RATTLE_DEBUG 0
#define RATTLE_TEST  0



class FixRattle : public FixShake {

 double ** vp;                      // for unconstrained velocity update 
 double dt, dtfvo2, dtfsq_full;     
 bool first_call;                   // for compatibility with SHAKE


 // Just for debugging
 int pid;
 FILE * dfp;


 public:
  FixRattle(class LAMMPS *, int, char **);
  ~FixRattle();
  virtual void post_force(int);
  virtual void initial_integrate(int);
  void vrattle(int m);
  void vrattle3(int m);
  void vrattle4(int m);
  void vrattle3angle(int m);
  void solve3x3exactly(const double a[][3], const double c[], double l[]);
  void update_v_half_nocons();
  int  setmask();

  virtual double memory_usage();
  virtual void grow_arrays(int);
  
  // Just for debugging
  bool check_molecule(double ** v, int m, bool checkr, bool checkv);
  bool check_constraints(double **v, bool checkr, bool checkv);
  void end_of_step();

};
}
#endif
#endif

